@@ -1,4 +1,3 @@
-
 from .. import haven_utils
 from .. import haven_results as hr
 from .. import haven_utils as hu
@@ -28,16 +27,18 @@ try:
     from IPython.core.display import Javascript, display, HTML
     from IPython.display import FileLink, FileLinks
     from ipywidgets.widgets.interaction import show_inline_matplotlib_plots
-except:
-    print('widgets not available...')
+except Exception:
+    print("widgets not available...")
 
 
 def get_dashboard(rm, vars=None, show_jobs=True, wide_display=False, enable_datatables=True):
     dm = DashboardManager(
-        rm, vars=vars, show_jobs=show_jobs, wide_display=wide_display, enable_datatables=enable_datatables)
+        rm, vars=vars, show_jobs=show_jobs, wide_display=wide_display, enable_datatables=enable_datatables
+    )
     dm.display()
     return dm
-    
+
+
 class DashboardManager:
     def __init__(self, rm, vars=None, show_jobs=True, wide_display=True, enable_datatables=True):
         self.rm_original = rm
@@ -50,27 +51,30 @@ class DashboardManager:
         self.wide_display = wide_display
         self.enable_datatables = enable_datatables
 
-        self.layout = widgets.Layout(width='100px')
-        self.layout_label = widgets.Layout(width='200px')
-        self.layout_dropdown = widgets.Layout(width='200px')
-        self.layout_button = widgets.Layout(width='200px')
+        self.layout = widgets.Layout(width="100px")
+        self.layout_label = widgets.Layout(width="200px")
+        self.layout_dropdown = widgets.Layout(width="200px")
+        self.layout_button = widgets.Layout(width="200px")
         self.t_savedir_base = widgets.Text(
-            value=str(self.vars.get('savedir_base') or rm.savedir_base),
-            layout=widgets.Layout(width='600px'),
-            disabled=False
+            value=str(self.vars.get("savedir_base") or rm.savedir_base),
+            layout=widgets.Layout(width="600px"),
+            disabled=False,
         )
 
         self.t_filterby_list = widgets.Text(
-            value=str(self.vars.get('filterby_list')),
-            layout=widgets.Layout(width='1200px'),
-            description='               filterby_list:',
-            disabled=False
+            value=str(self.vars.get("filterby_list")),
+            layout=widgets.Layout(width="1200px"),
+            description="               filterby_list:",
+            disabled=False,
         )
 
     def display(self):
         self.update_rm(display_meta=False)
 
-        header = widgets.Label(value="Loading Dashboard...", layout=self.layout_label,)
+        header = widgets.Label(
+            value="Loading Dashboard...",
+            layout=widgets.Layout(width="800px"),
+        )
         display(header)
 
         if self.enable_datatables:
@@ -84,12 +88,11 @@ class DashboardManager:
         main_out = widgets.Output()
         # Display tabs
         tab = widgets.Tab(children=[tables, plots, images, latex, share])
-        tab.set_title(0, 'Tables')
-        tab.set_title(1, 'Plots')
-        tab.set_title(2, 'Images')
-        tab.set_title(3, 'Latex')
-        tab.set_title(4, 'Share')
-        
+        tab.set_title(1, "Plots")
+        tab.set_title(0, "Tables")
+        tab.set_title(2, "Images")
+        tab.set_title(3, "Latex")
+        tab.set_title(4, "Share")
 
         with main_out:
             display(tab)
@@ -98,7 +101,6 @@ class DashboardManager:
             images.clear_output()
             latex.clear_output()
             share.clear_output()
-            
 
             # show tabs
             tables_tab(self, tables)
@@ -107,13 +109,15 @@ class DashboardManager:
             latex_tab(self, latex)
             share_tab(self, share)
 
-            header.value = f'Dashboard loaded (ver: {haven.__version__}).'
+            header.value = (
+                f"Dashboard loaded (ver: {haven.__version__}). "
+                + f'{len(self.rm_original.exp_list_all)} experiments selected from "{self.rm_original.savedir_base}"'
+            )
 
         display(main_out)
 
         if self.wide_display:
-            display(
-                HTML("<style>.container { width:100% !important; }</style>"))
+            display(HTML("<style>.container { width:100% !important; }</style>"))
 
         # This makes cell show full height display
         style = """
@@ -129,15 +133,16 @@ class DashboardManager:
         display(HTML(style))
 
     def update_rm(self, display_meta=True):
-        self.rm = hr.ResultManager(exp_list=self.rm_original.exp_list_all,
-                                   savedir_base=str(self.t_savedir_base.value),
-                                   filterby_list=hu.get_dict_from_str(
-                                       str(self.t_filterby_list.value)),
-                                   verbose=self.rm_original.verbose,
-                                   mode_key=self.rm_original.mode_key,
-                                   has_score_list=self.rm_original.has_score_list,
-                                   score_list_name=self.rm_original.score_list_name
-                                   )
+        self.rm = hr.ResultManager(
+            exp_list=self.rm_original.exp_list_all,
+            savedir_base=str(self.t_savedir_base.value),
+            filterby_list=hu.get_dict_from_str(str(self.t_filterby_list.value)),
+            verbose=self.rm_original.verbose,
+            mode_key=self.rm_original.mode_key,
+            has_score_list=self.rm_original.has_score_list,
+            score_list_name=self.rm_original.score_list_name,
+            job_scheduler=self.rm_original.job_scheduler,
+        )
 
         # if len(self.rm.exp_list) == 0:
         #     if self.rm.n_exp_all > 0:
@@ -153,9 +158,7 @@ class DashboardManager:
         #     return
         # else:
         if display_meta:
-            display('%d/%d experiments selected, using "filterby_list" from savedir_base "%s"' %
-                (len(self.rm.exp_list), len(self.rm.exp_list_all), self.vars.get('savedir_base')))
-
+            display(f'{len(self.rm.exp_list)}/{len(self.rm.exp_list_all)} experiments selected using "filterby_list"')
 
 
 def launch_jupyter():
@@ -168,39 +171,76 @@ def launch_jupyter():
     print()
 
 
-def create_jupyter(fname='example.ipynb',
-                   savedir_base='<path_to_saved_experiments>',
-                   overwrite=False, print_url=False,
-                   create_notebook=True):
-    print('Jupyter')
+def create_jupyter_file(
+    fname,
+    savedir_base,
+    overwrite=False,
+    print_url=False,
+):
 
-    if create_notebook and (overwrite or not os.path.exists(fname)):
-        cells = [main_cell(savedir_base), install_cell()]
-        os.makedirs(os.path.dirname(fname), exist_ok=True)
+    if overwrite or not os.path.exists(fname):
+        cells = [main_cell(savedir_base), sub_cell(), install_cell()]
+        if os.path.dirname(fname) != "":
+            os.makedirs(os.path.dirname(fname), exist_ok=True)
         save_ipynb(fname, cells)
-        print('- saved:', fname)
+        print("> Open %s to visualize results" % fname)
 
     if print_url:
         from notebook import notebookapp
+
         servers = list(notebookapp.list_running_servers())
         hostname = os.uname().nodename
 
         flag = False
         for i, s in enumerate(servers):
-            if s['hostname'] == 'localhost':
+            if s["hostname"] == "localhost":
                 continue
             flag = True
-            url = 'http://%s:%s/' % (hostname, s['port'])
-            print('- url:', url)
+            url = "http://%s:%s/" % (hostname, s["port"])
+            print("- url:", url)
 
         if flag == False:
-            print('a jupyter server was not found :(')
-            print(
-                'a jupyter server can be started using the script in https://github.com/ElementAI/haven .')
+            print("a jupyter server was not found :(")
+
+
+def sub_cell():
+    script = """
+# get table 
+rm.get_score_df().head()
+
+# get latex 
+# print(rm.get_latex_table(legend=['dataset'], metrics=['train_loss'], decimals=1, caption="Results", label='tab:results'))
+
+# get custom plots
+fig = rm.get_plot_all(
+                # order='metrics_by_groups',
+                # avg_across='runs',
+                y_metric_list=y_metrics, 
+                x_metric=x_metric,
+                # legend_fontsize=18,
+                # x_fontsize=20,
+                # y_fontsize=20,
+                # xtick_fontsize=20,
+                # ytick_fontsize=20,
+                # title_fontsize=24,
+                # legend_list=['model], 
+                # title_list = ['dataset'], 
+                # title_format='Dataset:{}',
+                # log_metric_list = ['train_loss'], 
+                # groupby_list = ['dataset'],
+                # map_ylabel_list=[{'train_loss':'Train loss'}],
+                # map_xlabel_list=[{'epoch':'Epoch'}],
+                # figsize=(15,5),
+                # plot_confidence=False,
+                # savedir_plots='%s' % (name)
+)
+          """
+    return script
 
 
 def main_cell(savedir_base):
-    script = ("""
+    script = (
+        """
 from haven import haven_jupyter as hj
 from haven import haven_results as hr
 from haven import haven_utils as hu
@@ -209,31 +249,50 @@ from haven import haven_utils as hu
 savedir_base = '%s'
 exp_list = None
 
+# get experiments from the exp config
+exp_config_fname = None
+if exp_config_fname:
+    config = hu.load_py(exp_config_fname)
+    exp_list = []
+    for exp_group in [
+        "example"
+                    ]:
+        exp_list += config.EXP_GROUPS[exp_group]
+
 # filter exps
-# e.g. filterby_list =[{'dataset':'mnist'}] gets exps with mnist
+
 filterby_list = None
+# filterby_list =[{'dataset':'mnist'}]
 
 # get experiments
-rm = hr.ResultManager(exp_list=exp_list, 
-                      savedir_base=savedir_base, 
+rm = hr.ResultManager(exp_list=exp_list,
+                      savedir_base=savedir_base,
                       filterby_list=filterby_list,
                       verbose=0,
-                      exp_groups=None
+                      exp_groups=None,
+                      job_scheduler='slurm'
                      )
 
-# launch dashboard
-# make sure you have 'widgetsnbextension' enabled; 
-# otherwise see README.md in https://github.com/ElementAI/haven
+# specify display parameters
 
-hj.get_dashboard(rm, vars(), wide_display=True)
-          """ % savedir_base)
+# groupby_list = ['dataset']
+# title_list = ['dataset']
+# legend_list = ['model']
+y_metrics = ['train_loss']
+x_metric = 'epoch'
+
+# launch dashboard
+hj.get_dashboard(rm, vars(), wide_display=False, enable_datatables=False)
+          """
+        % savedir_base
+    )
     return script
 
 
 def install_cell():
-    script = ("""
-    !pip install --upgrade git+https://github.com/ElementAI/haven
-          """)
+    script = """
+    !pip install --upgrade git+https://github.com/haven-ai/haven-ai
+          """
     return script
 
 
@@ -241,9 +300,8 @@ def save_ipynb(fname, script_list):
     import nbformat as nbf
 
     nb = nbf.v4.new_notebook()
-    nb['cells'] = [nbf.v4.new_code_cell(code) for code in
-                   script_list]
-    with open(fname, 'w') as f:
+    nb["cells"] = [nbf.v4.new_code_cell(code) for code in script_list]
+    with open(fname, "w") as f:
         nbf.write(nb, f)
 
 
@@ -254,24 +312,26 @@ def init_datatable_mode():
 
     # configure path to the datatables library using requireJS
     # that way the library will become globally available
-    display(Javascript("""
+    display(
+        Javascript(
+            """
         require.config({
             paths: {
                 DT: '//cdn.datatables.net/1.10.19/js/jquery.dataTables.min',
             }
         });
         $('head').append('<link rel="stylesheet" type="text/css" href="//cdn.datatables.net/1.10.19/css/jquery.dataTables.min.css">');
-    """))
+    """
+        )
+    )
 
     def _repr_datatable_(self):
         """Return DataTable representation of pandas DataFrame."""
         # classes for dataframe table (optional)
-        classes = ['table', 'table-striped', 'table-bordered']
+        classes = ["table", "table-striped", "table-bordered"]
 
         # create table DOM
-        script = (
-            f'$(element).html(`{self.to_html(index=True, classes=classes)}`);\n'
-        )
+        script = f"$(element).html(`{self.to_html(index=True, classes=classes)}`);\n"
 
         # execute jQuery to turn table into DataTable
         script += """
@@ -282,7 +342,7 @@ def init_datatable_mode():
 
                     $('#container').css( 'display', 'block' );
                     table.columns.adjust().draw();
-                    
+
                 })
             });
         """
@@ -290,6 +350,3 @@ def init_datatable_mode():
         return script
 
     pd.DataFrame._repr_javascript_ = _repr_datatable_
-
-
-
