@@ -4,7 +4,7 @@ import hashlib
 import itertools
 import json
 import tqdm
-
+import torchvision
 import pprint
 import os
 import pickle
@@ -15,6 +15,7 @@ import time
 import numpy as np
 import pylab as plt
 import torch
+import pandas as pd
 
 from .. import haven_img as hi
 from .image_utils import *
@@ -28,50 +29,76 @@ from PIL import Image
 def get_function_from_file():
     pass
 
+
 def create_command(base_command, args):
     """
     args is the parser
     """
     run_command = base_command
-    arg_keys= vars(args).keys()
+    arg_keys = vars(args).keys()
 
-    assert('exp_group_list' in arg_keys)
-    assert('exp_id' in arg_keys)
-    assert('run_jobs' in arg_keys)
+    assert "exp_group_list" in arg_keys
+    assert "exp_id" in arg_keys
+    assert "run_jobs" in arg_keys
 
     for a, v in vars(args).items():
-        if a == 'exp_group_list' or a == 'exp_id' or a == 'run_jobs' or a == 'reset':
-            print('argument: %s ignored...' % a)
+        if a == "exp_group_list" or a == "exp_id" or a == "run_jobs" or a == "reset":
+            print("argument: %s ignored..." % a)
             continue
 
-        run_command += ' --%s %s' % (a, v)
-    print('command: %s' % run_command)
-    return run_command 
+        run_command += " --%s %s" % (a, v)
+    print("command: %s" % run_command)
+    return run_command
+
 
 def mask_on_image(mask, image):
     from skimage.color import label2rgb
     from skimage.color import color_dict, colorlabel
     from skimage.segmentation import mark_boundaries
-    default_colors = ['red', 'blue', 'yellow', 'magenta', 'green',
-                     'indigo', 'darkorange', 'cyan', 'pink', 'yellowgreen']
+
+    default_colors = [
+        "red",
+        "blue",
+        "yellow",
+        "magenta",
+        "green",
+        "indigo",
+        "darkorange",
+        "cyan",
+        "pink",
+        "yellowgreen",
+    ]
 
     mask = mask.squeeze().astype(int)
-    image = hi.image_as_uint8(image) / 255.
+    image = hi.image_as_uint8(image) / 255.0
     labels = [l for l in np.unique(mask) if l < len(color_dict)]
-    colors =  (default_colors + list(color_dict.keys())[len(default_colors):])
-    colors =  np.array(colors)[labels]
-    
-    image_label_overlay = label2rgb(mask, image=f2l(image).squeeze().clip(0,1), 
-                                    colors=colors, bg_label=0, bg_color=None, kind='overlay')
+    colors = default_colors + list(color_dict.keys())[len(default_colors) :]
+    colors = np.array(colors)[labels]
+
+    image_label_overlay = label2rgb(
+        mask, image=f2l(image).squeeze().clip(0, 1), colors=colors, bg_label=0, bg_color=None, kind="overlay"
+    )
     return mark_boundaries(image_label_overlay, mask)
 
-def get_image(img, denorm=None, size=None, points=None, radius=10,
-              mask=None, heatmap=None):
-    return save_image(None, img, denorm, size, points, radius,
-               mask, heatmap, return_image=True)
 
-def save_image(fname, img, denorm=None, size=None, points=None, radius=10,
-               mask=None, heatmap=None, makedirs=True, return_image=False):
+def get_image(img, denorm=None, size=None, points=None, radius=10, mask=None, heatmap=None):
+    return save_image(None, img, denorm, size, points, radius, mask, heatmap, return_image=True)
+
+
+def save_image(
+    fname,
+    img,
+    denorm=None,
+    size=None,
+    points=None,
+    radius=10,
+    mask=None,
+    heatmap=None,
+    makedirs=True,
+    return_image=False,
+    nrow=8,
+    bbox_yxyx=None,
+):
     """Save an image into a file.
 
     Parameters
@@ -83,6 +110,12 @@ def save_image(fname, img, denorm=None, size=None, points=None, radius=10,
     makedirs : bool, optional
         If enabled creates the folder for saving the file, by default True
     """
+    if not isinstance(img, torch.Tensor):
+        if img.min() >= 0 and img.max() > 1:
+            img = img / 255.0
+        img = torch.as_tensor(img)
+    if img.ndim == 4:
+        img = torchvision.utils.make_grid(img, nrow=nrow)
     if denorm:
         img = denormalize(img, mode=denorm)
     if points is not None:
@@ -90,25 +123,22 @@ def save_image(fname, img, denorm=None, size=None, points=None, radius=10,
             img = torch.FloatTensor(img)
         img = img.squeeze()
         if img.ndim == 2:
-            img = img[None].repeat(3,1,1)
+            img = img[None].repeat(3, 1, 1)
         y_list, x_list = np.where(points.squeeze())
         c_list = []
         for y, x in zip(y_list, x_list):
             c_list += [points.squeeze()[y, x]]
-        img = hi.points_on_image(y_list, x_list, img, 
-                 radius=radius, c_list=c_list)
+        img = hi.points_on_image(y_list, x_list, img, radius=radius, c_list=c_list)
 
     if mask is not None:
         img = mask_on_image(mask, img)
 
-    
-
-    if img.dtype == 'uint8':
+    if img.dtype == "uint8":
         img = Image.fromarray(img)
     else:
         arr = f2l(t2n(img)).squeeze()
         # print(arr.shape)
-        if size is not None:  
+        if size is not None:
             arr = Image.fromarray(arr)
             arr = arr.resize(size)
             arr = np.array(arr)
@@ -119,7 +149,7 @@ def save_image(fname, img, denorm=None, size=None, points=None, radius=10,
 
     if fname is not None:
         dirname = os.path.dirname(fname)
-        if makedirs and dirname != '':
+        if makedirs and dirname != "":
             os.makedirs(dirname, exist_ok=True)
         img.save(fname)
 
@@ -141,6 +171,7 @@ def load_txt(fname):
         lines = f.readlines()
     return lines
 
+
 def save_txt(fname, lines):
     """Load the content of a txt file.
 
@@ -157,7 +188,7 @@ def save_txt(fname, lines):
     with open(fname, "w", encoding="utf-8") as f:
         for l in lines:
             f.writelines(l)
-    
+
 
 def torch_load(fname, map_location=None):
     """Load the content of a torch file.
@@ -205,13 +236,14 @@ def torch_save(fname, obj):
 
 class Parallel:
     """Class for run a function in parallel."""
+
     def __init__(self):
         """Constructor"""
         self.threadList = []
         self.count = 0
         self.thread_logs = []
 
-    def add(self, func,  *args):
+    def add(self, func, *args):
         """Add a function to run as a process.
 
         Parameters
@@ -221,42 +253,55 @@ class Parallel:
         args : list
             Arguments of the funtion to parallelize
         """
-        self.threadList += [
-            threading.Thread(target=func, name="thread-%d" % self.count,
-                             args=args)]
+        self.threadList += [threading.Thread(target=func, name="thread-%d" % self.count, args=args)]
         self.count += 1
 
     def run(self):
         """Run the added functions in parallel"""
-        for thread in tqdm.tqdm(self.threadList, desc='Starting threads', leave=False):
+        for thread in tqdm.tqdm(self.threadList, desc="Starting threads", leave=False):
             thread.daemon = True
             thread.start()
 
     def close(self):
         """Finish: wait for all the functions to finish"""
-        for thread in tqdm.tqdm(self.threadList, desc='Joining threads', leave=False):
+        for thread in tqdm.tqdm(self.threadList, desc="Joining threads", leave=False):
             thread.join()
 
 
 def pretty_print_df(df):
     # wrap text for prettiness
     for c in df.columns:
-        if df[c].dtype == 'O':
+        if df[c].dtype == "O":
             # df[c] = df[c].str.wrap(wrap_size)
             df[c] = df[c].apply(pprint.pformat)
-    return df 
+    return df
 
-def flatten_column(result_dict):
+
+def flatten_column(result_dict, flatten_list=False):
     new_dict = {}
 
     for k, v in result_dict.items():
-        new_dict.update(flatten_dict(k, v))
+        if isinstance(v, list) and flatten_list:
+            list_dict = {}
+            for vi in v:
+                flat = flatten_dict(k, vi)
+                for f1, f2 in flat.items():
+                    if f1 in list_dict:
+                        list_dict[f1] += "_" + str(f2)
+                    else:
+                        list_dict[f1] = str(f2)
+            # print()
+        else:
+            list_dict = flatten_dict(k, v)
+
+        new_dict.update(list_dict)
 
     result_dict = new_dict
     return result_dict
 
+
 def sort_df_columns(table, also_first=[]):
-    first = ['exp_id', 'job_state', 'job_id',  'restarts', 'started_at']
+    first = ["exp_id", "job_state", "job_id", "restarts", "started_at"]
     first += also_first
     col_list = []
     for col in first:
@@ -266,9 +311,10 @@ def sort_df_columns(table, also_first=[]):
         if col in first:
             continue
         col_list += [col]
-        
+
     return table[col_list]
 
+
 def subprocess_call(cmd_string):
     """Run a terminal process.
 
@@ -282,13 +328,12 @@ def subprocess_call(cmd_string):
     [type]
         Error code or 0 if no error happened
     """
-    return subprocess.check_output(
-        shlex.split(cmd_string), shell=False).decode("utf-8")
+    return subprocess.check_output(shlex.split(cmd_string), shell=False, stderr=subprocess.STDOUT).decode("utf-8")
 
 
 def copy_code(src_path, dst_path, verbose=1):
     """Copy the code.
-    
+
     Typically, when you run an experiment, first you copy the code used to the
     experiment folder. This function copies the code using rsync terminal
     command.
@@ -307,7 +352,7 @@ def copy_code(src_path, dst_path, verbose=1):
     ValueError
         [description]
     """
-    time.sleep(.5)  # TODO: Why? Why?
+    time.sleep(0.5)  # TODO: Why? Why?
 
     if verbose:
         print("  > Copying code from %s to %s" % (src_path, dst_path))
@@ -316,9 +361,18 @@ def copy_code(src_path, dst_path, verbose=1):
     os.makedirs(dst_path, exist_ok=True)
 
     # Define the command for copying the code using rsync
-    rsync_code = "rsync -av -r -q  --delete-before --exclude='.tmp/' --exclude='.git/' " \
-                 " --exclude='*.pyc' --exclude='__pycache__/' %s %s" % (
-                     src_path, dst_path)
+    if os.path.exists(os.path.join(src_path, ".havenignore")):
+        rsync_code = (
+            "rsync -av -r -q  --delete-before --exclude='.*'  --exclude-from=%s \
+                        --exclude '__pycache__/' %s %s"
+            % (os.path.join(src_path, ".havenignore"), src_path, dst_path)
+        )
+    else:
+        rsync_code = (
+            "rsync -av -r -q  --delete-before --exclude='.*' \
+                        --exclude '__pycache__/' %s %s"
+            % (src_path, dst_path)
+        )
 
     # Run the command in the terminal
     try:
@@ -326,7 +380,7 @@ def copy_code(src_path, dst_path, verbose=1):
     except subprocess.CalledProcessError as e:
         raise ValueError("Ping stdout output:\n", e.output)
 
-    time.sleep(.5)  # TODO: Why?
+    time.sleep(0.5)  # TODO: Why?
 
 
 def zipdir(src_dirname, out_fname, include_list=None):
@@ -343,9 +397,10 @@ def zipdir(src_dirname, out_fname, include_list=None):
         default None
     """
     import zipfile  # TODO: Move to the beggining of the file
+
     # TODO: Do we need makedirs?
     # Create the zip file
-    zipf = zipfile.ZipFile(out_fname, 'w', zipfile.ZIP_DEFLATED)
+    zipf = zipfile.ZipFile(out_fname, "w", zipfile.ZIP_DEFLATED)
 
     # ziph is zipfile handle
     for root, dirs, files in os.walk(src_dirname):
@@ -353,7 +408,7 @@ def zipdir(src_dirname, out_fname, include_list=None):
             # Descard files if needed
             if include_list is not None and file not in include_list:
                 continue
-            
+
             abs_path = os.path.join(root, file)
             rel_path = fname_parent(abs_path)  # TODO: fname_parent not defined
             print(rel_path)
@@ -381,11 +436,10 @@ def zip_score_list(exp_list, savedir_base, out_fname, include_list=None):
         # Get the experiment id
         exp_id = hash_dict(exp_dict)
         # Zip folder
-        zipdir(os.path.join(savedir_base, exp_id),
-               out_fname, include_list=include_list)
+        zipdir(os.path.join(savedir_base, exp_id), out_fname, include_list=include_list)
 
 
-def time_to_montreal(fname=None, timezone='US/Eastern'):
+def time_to_montreal(fname=None, timezone="US/Eastern"):
     """Get time in Montreal zone.
 
     Returns
@@ -394,15 +448,18 @@ def time_to_montreal(fname=None, timezone='US/Eastern'):
         Current date at the selected timezone in string format
     """
     # Get time
-    os.environ['TZ'] = timezone
-    time.tzset()
+    os.environ["TZ"] = timezone
+    try:
+        time.tzset()
+    except:
+        pass
     if fname:
         tstamp = os.path.getctime(fname)
     else:
         tstamp = time.time()
-        
-    time_str = datetime.fromtimestamp(tstamp).strftime('%I:%M %p (%b %d)')
-    
+
+    time_str = datetime.fromtimestamp(tstamp).strftime("%I:%M %p (%b %d)")
+
     return time_str
 
 
@@ -419,7 +476,7 @@ def time2mins(time_taken):
     float
         Minutes
     """
-    return time_taken / 60.
+    return time_taken / 60.0
 
 
 def n2t(x, dtype="float"):  # TODO: dtype is not used!!
@@ -460,7 +517,7 @@ def t2n(x):
     """
     try:
         x = x.detach().cpu().numpy()
-    except:
+    except Exception:
         x = x
 
     return x
@@ -499,15 +556,15 @@ def l2f(X):
 def f2l(X):
     """Move the channels from the first dimension to the last dimension.
 
-`   Parameters
-    ----------
-    X : Numpy array
-        Tensor with the channel dimension at the first dimension
+    `   Parameters
+        ----------
+        X : Numpy array
+            Tensor with the channel dimension at the first dimension
 
-    Returns
-    -------
-    Numpy array
-        X transformed with the channel dimension at the last dimension
+        Returns
+        -------
+        Numpy array
+            X transformed with the channel dimension at the last dimension
     """
     if X.ndim == 3 and (X.shape[2] == 3 or X.shape[2] == 1):
         return X
@@ -523,7 +580,7 @@ def f2l(X):
     return X
 
 
-def n2p(image):  #TODO: Create p2n function and use it in get_image()
+def n2p(image):  # TODO: Create p2n function and use it in get_image()
     """Numpy image to PIL image.
 
     Parameters
@@ -539,7 +596,7 @@ def n2p(image):  #TODO: Create p2n function and use it in get_image()
     image = f2l(image.squeeze())
     if image.max() <= 1:
         image = image * 255
-    return Image.fromarray(image.astype('uint8'))
+    return Image.fromarray(image.astype("uint8"))
 
 
 def _denorm(image, mu, var, bgr2rgb=False):
@@ -581,7 +638,7 @@ def denormalize(img, mode=0):  # TODO: Remove the default value or set to a vali
         Input image to denormalize
     mode : int or str, optional
         Predefined denormalizations, by default 0
-        If 1 or 'rgb'... 
+        If 1 or 'rgb'...
         If 2 or 'brg'...,
         If 3 or 'basic'...
         Else do nothing
@@ -707,18 +764,17 @@ def show_image(fname):  # TODO: Why the input is a filename instead of an image?
     nrows = 1
     height = 12
     width = 12
-    fig, axs = plt.subplots(nrows=nrows, ncols=ncols,
-                            figsize=(ncols*width, nrows*height))
-    if not hasattr(axs, 'size'):  #TODO: What is this?
+    fig, axs = plt.subplots(nrows=nrows, ncols=ncols, figsize=(ncols * width, nrows * height))
+    if not hasattr(axs, "size"):  # TODO: What is this?
         axs = [[axs]]
 
     for i in range(ncols):
         img = plt.imread(fname)
         axs[0][i].imshow(img)
         axs[0][i].set_axis_off()
-        axs[0][i].set_title('%s' % (fname))
+        axs[0][i].set_title("%s" % (fname))
 
-    plt.axis('off')
+    plt.axis("off")
     plt.tight_layout()
 
     plt.show()
@@ -830,6 +886,7 @@ def as_double_list(v):
 
     return v
 
+
 def ignore_duplicates(list_of_dicts):
     # ensure no duplicates in exp_list
     dict_list = []
@@ -843,16 +900,33 @@ def ignore_duplicates(list_of_dicts):
             dict_list += [data_dict]
     return dict_list
 
+
+def filter_duplicates(list_of_dicts):
+    # ensure no duplicates in exp_list
+    tmp_list = []
+    hash_list = set()
+    for data_dict in list_of_dicts:
+        dict_id = hash_dict(data_dict)
+        if dict_id in hash_list:
+            continue
+        else:
+            hash_list.add(dict_id)
+        tmp_list += [data_dict]
+
+    return tmp_list
+
+
 def check_duplicates(list_of_dicts):
     # ensure no duplicates in exp_list
     hash_list = set()
     for data_dict in list_of_dicts:
         dict_id = hash_dict(data_dict)
         if dict_id in hash_list:
-            raise ValueError('duplicated dictionary detected:\n%s' %pprint.pformat(data_dict))
+            raise ValueError("duplicated dictionary detected:\n%s" % pprint.pformat(data_dict))
         else:
             hash_list.add(dict_id)
 
+
 def load_py(fname):
     """[summary]
 
@@ -871,12 +945,12 @@ def load_py(fname):
     from importlib import import_module
 
     if not os.path.exists(fname):
-        raise ValueError('%s not found...' % fname)
+        raise ValueError("%s not found..." % fname)
 
     sys.path.append(os.path.dirname(fname))
 
-    name = os.path.split(fname)[-1].replace('.py','')
-    module = import_module(name) 
+    name = os.path.split(fname)[-1].replace(".py", "")
+    module = import_module(name)
     reload(module)
     sys.path.pop()
 
@@ -884,69 +958,138 @@ def load_py(fname):
 
 
 def get_exp_list_from_ids(exp_id_list, savedir_base):
-    exp_list = [] 
+    exp_list = []
     for exp_id in exp_id_list:
-        exp_list += [load_json(os.path.join(savedir_base, exp_id, 
-                        'exp_dict.json'))]
+        exp_list += [load_json(os.path.join(savedir_base, exp_id, "exp_dict.json"))]
     return exp_list
 
 
 def flatten_dict(key_name, v_dict):
     if not isinstance(v_dict, dict):
         return {key_name: v_dict}
-    
+
     leaf_dict = {}
     for k in v_dict:
-        if key_name != '':
+        if key_name != "":
             k_new = key_name + "." + k
         else:
             k_new = k
         leaf_dict.update(flatten_dict(key_name=k_new, v_dict=v_dict[k]))
-        
+
     return leaf_dict
 
 
-def get_diff_columns(df, min_threshold=2, max_threshold='auto'):
+def get_diff_hparam(exp_list):
+    df = pd.DataFrame([flatten_column(e) for e in exp_list])
+    return get_diff_columns(df, min_threshold=2, max_threshold="auto")
+
+
+def get_diff_columns(df, min_threshold=2, max_threshold="auto"):
     df.reset_index()
-    if max_threshold == 'auto':
+    if max_threshold == "auto":
         max_threshold = df.shape[0]
-    
+
     if max_threshold < 0:
         max_threshold = df.shape[0] + max_threshold
-    
+
     column_count = []
-    
+
     for column in df.columns:
         _set = set([str(v) for v in df[column].values])
         column_count.append(len(_set))
     indices = np.arange(len(df.columns))
-    
+
     column_count = np.array(column_count)
-    
+
     indices = indices[(column_count >= min_threshold) & (column_count <= max_threshold)]
     diff_columns = [df.columns[i] for i in indices]
-    
+
     return diff_columns
 
 
-def collate_fn(batch, mode='list'):
-    if mode == 'list':
+def collate_fn(batch, mode="list"):
+    if mode == "list":
         batch_dict = {}
         for k in batch[0]:
             batch_dict[k] = []
             for i in range(len(batch)):
-                
+
                 batch_dict[k] += [batch[i][k]]
 
-        return batch_dict 
-        
-    elif mode == 'default':
+        return batch_dict
+
+    elif mode == "default":
         return torch.utils.data.dataloader.default_collate(batch)
 
+
 def timeit(func, n_times=10, **args):
     for i in range(n_times):
-        if i ==1:
+        if i == 1:
             s = time.time()
         func(**args)
 
-    print('time:', (time.time()-s)/(n_times-1))
\ No newline at end of file
+    print("time:", (time.time() - s) / (n_times - 1))
+
+
+# Create Helper
+def make_binary_linear(n, d, margin, separable=True, seed=42):
+    np.random.seed(seed)
+    labels = [-1, 1]
+    w = np.random.randn(d)
+    w /= np.linalg.norm(w)
+    p = np.random.randn(d - 1)
+    l = (-p @ w[: d - 1]) / w[-1]
+    p = np.append(p, [l])
+    v0 = p - margin * w
+    v1 = p + margin * w
+    yv = np.copy(labels)
+    # Start generating points with rejection sampling
+    X = []
+    y = []
+    for i in range(n - 2):
+        s = 1
+        label = np.random.choice(labels)
+        # Generate a random point with mean at the center
+        xi = np.random.randn(d)
+        xi = (xi / np.linalg.norm(xi)) * s
+        dist = xi @ w
+        while dist * label <= margin:
+            u = v0 - v1 if label == -1 else v1 - v0
+            u /= np.linalg.norm(u)
+            xi = xi + u
+            xi = (xi / np.linalg.norm(xi)) * s
+            dist = xi @ w
+        X.append(xi)
+        y.append(label)
+    X = np.array(X).astype(float)
+    y = np.array(y)
+    # shuffle
+    ind = np.random.permutation(n - 2)
+    X = X[ind]
+    y = y[ind]
+    # Put the support vectors at the beginning
+    X = np.r_[np.array([v0, v1]), X]
+    y = np.r_[np.array(yv), y]
+    if not separable:
+        flip_ind = np.random.choice(n, int(n * 0.01))
+        y[flip_ind] = -y[flip_ind]
+    y[y == -1] = 0
+    X = np.c_[np.ones(n), X]
+    return X, y
+
+
+def get_split_torch_dataset(X, y, split):
+    from sklearn.model_selection import train_test_split
+
+    splits = train_test_split(X, y, test_size=0.2, shuffle=False, random_state=42)
+    X_train, X_test, Y_train, Y_test = splits
+    X_train, X_test = torch.FloatTensor(X_train), torch.FloatTensor(X_test)
+    Y_train, Y_test = torch.LongTensor(Y_train), torch.LongTensor(Y_test)
+
+    if split == "train":
+        dataset = torch.utils.data.TensorDataset(X_train, Y_train)
+    elif split == "val":
+        dataset = torch.utils.data.TensorDataset(X_test, Y_test)
+    dataset.n_input = X.shape[1]
+
+    return dataset
