@@ -5,24 +5,25 @@ def get_dict_from_str(string):
     if string is None:
         return string
 
-    if string == 'None':
+    if string == "None":
         return None
 
-    if string == '':
+    if string == "":
         return None
-        
+
     return ast.literal_eval(string)
 
+
 def get_list_from_str(string):
     if string is None:
         return string
 
-    if string == 'None':
+    if string == "None":
         return None
 
-    string = string.replace(' ','').replace(']', '').replace('[', '').replace('"', '').replace("'", "")
+    string = string.replace(" ", "").replace("]", "").replace("[", "").replace('"', "").replace("'", "")
 
-    if string == '':
+    if string == "":
         return None
 
-    return string.split(',')
\ No newline at end of file
+    return string.split(",")
